const jsdom = require('jsdom');
const { JSDOM } = jsdom;

const through = require('through2');
const prism = require('prismjs');

const loadLanguages = require('prismjs/components/');
loadLanguages();

function decodeEntities(encodedString) {
    var translate_re = /&(nbsp|amp|quot|lt|gt);/g;
    var translate = {
        "nbsp":" ",
        "amp" : "&",
        "quot": "\"",
        "lt"  : "<",
        "gt"  : ">"
    };
    return encodedString.replace(translate_re, function(match, entity) {
        return translate[entity];
    }).replace(/&#(\d+);/gi, function(match, numStr) {
        var num = parseInt(numStr, 10);
        return String.fromCharCode(num);
    });
}

const parse = html => {
    const dom = new JSDOM(html);

    const code_blocks = dom.window.document.querySelectorAll(`code[class^="language-"]`);

    code_blocks.forEach(code_block => {
        const langPrefixRegex = /\blang(?:uage)?-([\w-]+)\b/i;

        let match = code_block.classList.toString().match(langPrefixRegex);
        if (match) {
            let language = match[1];
            code_block.innerHTML = prism.highlight(decodeEntities(code_block.innerHTML), Prism.languages[language])
        }
    });

    return dom.serialize();
}

module.exports = (options) => {
    return through.obj(function (file, encoding, done) {
        if (file.isNull()) {
            return done(null, file);
        }

        if (file.isStream()) {
            // streams not supported
            return done();
        }

        try {
            const text = file.contents.toString();
            file.contents = Buffer.from(parse(text));

        } catch (error) {
            // error
        }

        done(null, file);
    })
}
